__version__ = '0.1.2'

from .conf import *
from .client import *
from .dtos import *
from .fields import *
from .exceptions import *
from .query import *
from .signer import *


VERSION = tuple(__version__.split('.'))



import os
import json
from base64 import b64decode, b64encode
from typing import Sequence, TYPE_CHECKING, Tuple
from functools import cached_property
from importlib import import_module
from contextlib import contextmanager
from pathlib import Path

from .exceptions import SignerNotLoaded, SignerPrivateKeyNotRead
from .conf import SignerSettings

if TYPE_CHECKING:
    from wc_bankid_nbu.contrib.eu_sign.EUSignCP import EU_INTERFACE


__all__ = 'update_env', 'Signer',


@contextmanager
def update_env(library_path: str):
    old = os.environ.get('LD_LIBRARY_PATH')
    os.environ['LD_LIBRARY_PATH'] = library_path

    yield

    os.environ['LD_LIBRARY_PATH'] = old


class Signer:
    settings: SignerSettings
    interface: 'EU_INTERFACE'
    loaded: bool
    _private_key_data: dict

    def __init__(self, settings: SignerSettings):
        self.settings = settings
        self.loaded = False
        self.interface = None

    @cached_property
    def module(self):
        return import_module(self.settings.MODULE)

    def env(self):
        return update_env(str(self.settings.C_PATH))

    def raise_for_unloaded(self):
        if not self.loaded:
            raise SignerNotLoaded()

    def configure(self):
        self.interface.SetFileStoreSettings({
            'szPath': self.settings.FILE_STORE_PATH,
            'bCheckCRLs': False,
            'bOwnCRLsOnly': True,
            'bAutoRefresh': False,
            'bFullAndDeltaCRLs': True,
            'bAutoDownloadCRLs': False,
            'bSaveLoadedCerts': True,
            'dwExpireTime': 3600,
        })

    def load(self):
        with self.env():
            self.module.EULoad()
            self.interface = self.module.EUGetInterface()

        self.interface.Initialize()
        self.configure()
        self.loaded = True

    def unload(self):
        with self.env():
            self.loaded = False
            self.interface.Finalize()
            self.module.EUUnload()

    @contextmanager
    def signer(self, unload: bool = True):
        if not self.loaded:
            self.load()

        yield self

        if unload:
            self.unload()

    def read_private_key(self):
        self.raise_for_unloaded()

        if self.interface.IsPrivateKeyReaded():
            return self._private_key_data

        with open(self.settings.PRIVATE_KEY_PATH, 'rb') as f:
            self._private_key_data = {}
            key = f.read()
            self.interface.ReadPrivateKeyBinary(
                key,
                len(key),
                self.settings.PRIVATE_KEY_PASS,
                self._private_key_data
            )

            if not self.interface.IsPrivateKeyReaded():
                raise SignerPrivateKeyNotRead()

        return self._private_key_data

    def _get_certificate(self, filename: str, b64: bool = False):
        with open(filename, 'rb') as f:
            data = f.read()

            if not b64:
                return data

            return b64encode(data).decode('ascii')

    def _fstore_path(self, name: str) -> str:
        return Path(self.settings.FILE_STORE_PATH) / name

    def get_distribution_certificate(self, b64: bool = False):
        return self._get_certificate(
            self._fstore_path(self.settings.CERTIFICATES_DISTRIBUTION_NAME),
            b64=b64,
        )

    def get_sign_certificate(self, b64: bool = False):
        return self._get_certificate(
            self._fstore_path(self.settings.CERTIFICATES_SIGN_NAME),
            b64=b64,
        )

    def decrypt(self, cert: str, encrypted: str) -> Tuple[dict, dict]:
        self.raise_for_unloaded()

        info = {}
        data: Sequence[bytes] = []
        result = []
        decoded = b64decode(encrypted)
        decoded_cert = b64decode(cert)

        self.interface.DevelopDataEx(
            encrypted, decoded, len(decoded), decoded_cert, len(decoded_cert),
            data, info
        )
        self.interface.VerifyDataInternal(
            b64encode(data[0]), data[0], len(data[0]),
            result, info
        )
        self.interface.GetDataFromSignedData(
            b64encode(data[0]), data[0], len(data[0]),
            result
        )

        return json.loads(result[0].decode()), info



from enum import Enum


__all__ = (
    'TAddress', 'TDocument', 'TScan',
    'FPerson', 'FAddress', 'FDocument', 'FScan',
)


class TypesEnum(str, Enum):
    pass


class TAddress(TypesEnum):
    FACTUAL = 'factual'
    JURIDICAL = 'juridical'


class TDocument(TypesEnum):
    PASSPORT_BOOK = 'passport'
    PASSPORT_ID = 'idpassport'
    PASSPORT_INTERNATIONAL = 'zpassport'
    IDENTITY = 'ident'


class TScan(TypesEnum):
    PASSPORT_BOOK = 'passport'
    PASSPORT_ID = 'idpassport'
    PASSPORT_INTERNATIONAL = 'zpassport'
    RNOKPP = 'inn'
    PHOTO = 'personalPhoto'


class FieldsEnum(str, Enum):
    pass


class FPerson(FieldsEnum):
    # * Прізвище
    LAST_NAME = 'lastName'
    # * Ім’я
    FIRST_NAME = 'firstName'
    # * По батькові
    MIDDLE_NAME = 'middleName'

    # Номер телефону
    # 380ХХХХХХХХХ - де Х може приймати тільки цифрове значення.
    PHONE = 'phone'
    # Електронна адреса
    EMAIL = 'email'

    # Реєстраційний номер облікової картки платника податків, номер
    # (та за наявності - серію) паспорта громадянина України, в якому
    # проставлено відмітку про відмову від прийняття реєстраційного
    # номера облікової картки платника податків, чи номер паспорта
    # із записом про відмову від прийняття реєстраційного номера облікової
    # картки платника податків в електронному безконтактному носії.
    # * Заповнюється відповідно до вимог законодавства України
    RNOKPP = 'inn'

    # Унікальний ідентифікатор клієнта в банку.
    # У випадку якщо банк не має такого ідентифікатора, можливо
    # вказати значення ключа іnn або серію й номер паспорта.
    CLIENT_ID = 'clId'
    # Статичний текст з інформацією про надані дані банком щодо користувача,
    # дата і час надання.
    # "Інформація надана засобами Системи BankID НБУ dd.mm.yyyy hh.mm"
    CLIENT_ID_TEXT = 'clIdText'

    # Дата народження.
    # * dd.mm.yyyy
    BIRTH_DAY = 'birthDay'
    # Стать
    # * Можливі значення: латинська літера M – чоловіча або F ‒ жіноча
    SEX = 'sex'
    # Соціальний статус
    # Наприклад: “студент”, “пенсіонер”, “тимчасово безробітний”, “працюючий”,
    # “нерегулярна зайнятість”.
    SOCIAL_STATUS = 'socStatus'

    # Ознака, чи визначена фізична особа банком-повіреним такою,
    # що належить до категорії PEPs (публічні особи, близькі, пов’язані).
    # Можливі значення: 1 – так, 0 – ні.
    IS_PEPS = 'flagPEPs'
    # Ознака, чи визначена фізична особа банком-повіреним такою, що включена
    # до переліку осіб, пов’язаних зі здійсненням терористичної діяльності або
    # щодо яких застосовано міжнародні санкції.
    # Можливі значення: 1 – так, 0 – ні.
    IS_TERRORIST = 'flagPersonTerror'
    # Ознака, чи визначена фізична особа банком-повіреним такою,
    # що включена до переліку осіб, щодо яких застосовані персональні,
    # спеціальні економічні та інші обмежувальні заходи (санкції),
    # санкції РНБОУ.
    # Можливі значення: 1 – так, 0 – ні.
    IS_RESTRICTED = 'flagRestriction'
    # Ознака, чи присвоєний клієнту банком-повіреним (неприйнятно) високий
    # рівень ризику ПВК/ФТ.
    # Можливі значення: 1 – так, 0 – ні.
    IS_HIGH_RISK = 'flagTopLevelRisk'
    # Ознака, чи визначена фізична особа банком-повіреним такою,
    # що є резидентом України.
    # Можливі значення: 1 – так, 0 – ні.
    IS_UA_RESIDENT = 'uaResident'


class FAddress(FieldsEnum):
    # Країна
    # * Двозначний літерний код країни за стандартом ISO_3166-1 (alfa-2).
    # Наприклад: UA
    COUNTRY = 'country'
    # Область.
    # * Якщо адреса користувача не передбачає наявності області,
    # відповідне значення не передається у відповіді абонента-ідентифікатора.
    REGION = 'state'
    # Район.
    # * Якщо адреса користувача не передбачає наявності району,
    # відповідне значення не передається у відповіді абонентаідентифікатора.
    DISTRICT = 'area'
    # * Місто
    CITY = 'city'
    # * Вулиця
    STREET = 'street'
    # * Номер будинку
    STREET_NUMBER = 'houseNo'
    # Номер квартири.
    # * Якщо адреса користувача не передбачає наявності номеру квартири,
    # відповідне значення не передається у відповіді абонентаідентифікатора.
    FLAT_NUMBER = 'flatNo'


class FDocument(FieldsEnum):
    # * Назва документу
    TYPE = 'typeName'
    # * Серія документа
    # (для типу idpassport - не
    # заповнюється).
    SERIES = 'series'
    # * Номер документа
    NUMBER = 'number'
    # * Яким органом видано документ
    ISSUED_BY = 'issue'
    # Країна видачі документа
    # Двозначний літерний код країни за стандартом ISO_3166-1 (alfa-2).
    # Наприклад: UA
    ISSUED_BY_COUNTRY = 'issueCountryIso2'
    # Дата видачі документу
    # * dd.mm.yyyy
    ISSUED_AT = 'dateIssue'
    # Термін дії (для типу passport - не заповнюється)
    # * dd.mm.yyyy
    EXPIRED_AT = 'dateExpiration'


class FScan(FieldsEnum):
    # Файл сканованої копії документу.
    # Закодований по стандарту BASE64.
    # Якщо скан-копія знаходиться в архіві, то файл скан-копій
    # обов’язково повинен мати розширення (наприклад: pdf, jpg, png, bmp).
    # Рекомендована роздільна здатність скан-копії не менше 200 DPI.
    DATA = 'scanFile'
    # У випадку, якщо файли об’єднанні або заархівовані у zip форматі,
    # то зазначається розширення zip.
    # Розширення файлу, який знаходиться в значенні ключа scanFile.
    EXTENSION = 'extension'
    # Дата створення скан-копії документа
    # dd.mm.yyyy
    CREATED_AT = 'dateCreate'



from typing import List, Optional, TypedDict


__all__ = 'PersonalInfoDTO', 'PersonalDataDTO', 'PersonalDataResult',


class PersonalInfoDTO(TypedDict):
    last_name: Optional[str]
    first_name: Optional[str]
    middle_name: Optional[str]

    phone: Optional[str]
    email: Optional[str]

    rnokpp: Optional[str]

    client_id: Optional[str]
    client_id_text: Optional[str]

    birth_day: Optional[str]
    sex: Optional[str]
    social_status: Optional[str]

    is_peps: Optional[bool]
    is_terrorist: Optional[bool]
    is_restricted: Optional[bool]
    is_high_risk: Optional[bool]
    is_ua_resident: Optional[bool]


class AddressDTO(TypedDict):
    type: Optional[str]
    country: Optional[str]
    state: Optional[str]
    area: Optional[str]
    city: Optional[str]
    street: Optional[str]
    street_number: Optional[str]
    flat_number: Optional[str]


class PersonalDataDTO(TypedDict):
    info: PersonalInfoDTO
    addresses: List[AddressDTO]

class PersonalDataResult(TypedDict):
    data: PersonalDataDTO
    certificate: dict



from functools import cached_property
from requests import request

from .query import QueryBuilder
from .signer import Signer
from .dtos import PersonalDataResult, PersonalDataDTO
from .transforms import personal_data_response_to_dto


__all__ = 'APIClient', 'TestAPIClient'


def urlprop(postfix: str):
    return cached_property(lambda self: self.BASE_URL + postfix)


class APIClient:
    BASE_URL: str = 'https://id.bank.gov.ua/v1'
    AUTHORIZATION_URL: str = urlprop('/bank/oauth2/authorize')
    ACCESS_TOKEN_URL: str = urlprop('/bank/oauth2/token')
    REFRESH_TOKEN_URL: str = urlprop('/bank/oauth2/refresh_token')
    PERSONAL_DATA_URL: str = urlprop('/bank/resource/client')

    signer: Signer

    def __init__(self, signer: Signer = None):
        assert signer is not None, 'You must provide `signer`.'

        self.signer = signer

    def parse_personal_data(self, response: dict) -> PersonalDataDTO:
        return personal_data_response_to_dto(response)

    def get_personal_data(
        self,
        query: QueryBuilder,
        access_token: str,
    ) -> PersonalDataResult:
        with self.signer.signer() as signer:
            parameters = {
                **query.to_foreign(),
                'cert': signer.get_distribution_certificate(b64=True)
            }

            response = self.make_request(
                self.PERSONAL_DATA_URL,
                headers={
                    'Accept': 'application/json',
                    'Authorization': f'Bearer {access_token}',
                    'Content-Type': 'application/json',
                },
                json=parameters,
                method='POST',
            ).json()

            signer.read_private_key()
            data, certificate = signer.decrypt(
                response['cert'], response['customerCrypto']
            )

        return {
            'data': self.parse_personal_data(data),
            'certificate': certificate,
        }

    def make_request(self, url, method='GET', *args, **kwargs):
        kwargs.setdefault('headers', {})

        response = request(method, url, *args, **kwargs)
        response.raise_for_status()
        return response


class TestAPIClient(APIClient):
    BASE_URL: str = 'https://testid.bank.gov.ua/v1'



from typing import Any, Sequence, TypeVar
from copy import deepcopy


__all__ = 'QueryBuilder', 'FieldsDataQuery', 'PersonalPhysicalDataQuery'


T = TypeVar('T')


class QueryBuilder:
    query: dict

    def __init__(self, query: dict = {}):
        self.query = query

    def clone(self, query: dict):
        return deepcopy(query)

    def nest(self) -> 'QueryBuilder':
        return self.__class__(query=self.clone(self.query))

    def to_foreign(self) -> Any:
        raise NotImplementedError(
            'Implement final query transformation in `to_foreign` method.'
        )


class FieldsDataQuery(QueryBuilder):
    def _set(self: T, *path: Sequence[str], fields: Sequence[str]) -> T:
        s = self.nest()
        q: dict = s.query
        last = path[-1]

        for key in path:
            if key not in q:
                if key is last:
                    q[key] = q.get(key) or set()
                else:
                    q[key] = q.get(key) or {}

            q = q[key]

        q: set = q
        q.update(fields)

        return s


class PersonalPhysicalDataQuery(FieldsDataQuery):
    def fields(self: T, *fields: Sequence[str]) -> T:
        return self._set('fields', fields=fields)

    def address(self: T, type: str, *fields: Sequence[str]) -> T:
        return self._set('addresses', type, 'fields', fields=fields)

    def document(self: T, type: str, *fields: Sequence[str]) -> T:
        return self._set('documents', type, 'fields', fields=fields)

    def scan(self: T, type: str, *fields: Sequence[str]) -> T:
        return self._set('scans', type, 'fields', fields=fields)

    def _collect_typed_fields(self, query_part: dict) -> Sequence[dict]:
        return [
            {'type': type, 'fields': list(data['fields'])}
            for type, data in query_part.items()
        ]

    def to_foreign(self) -> dict:
        result = {
            'type': 'physical',
            'fields': list(self.query.get('fields', [])),
            'addresses': self._collect_typed_fields(self.query.get('addresses', {})),
            'documents': self._collect_typed_fields(self.query.get('documents', {})),
            'scans': self._collect_typed_fields(self.query.get('scans', {})),
        }

        return result



from dataclasses import dataclass
from pathlib import Path


__all__ = 'LIB_PATH', 'SignerSettings',

LIB_PATH = Path(__file__).parent


@dataclass
class SignerSettings:
    PRIVATE_KEY_PATH: str
    PRIVATE_KEY_PASS: str

    FILE_STORE_PATH: str = '/data/certificates'
    C_PATH: str = LIB_PATH / 'contrib' / 'eu_sign' / 'linux' / '64'
    MODULE: str = 'wc_bankid_nbu.contrib.eu_sign.EUSignCP'

    CERTIFICATES_SIGN_NAME: str = 'EU-5B63D88375D92018040000002E3D0000B1950000.cer'
    CERTIFICATES_DISTRIBUTION_NAME: str = 'EU-5B63D88375D92018040000002E3D0000B2950000.cer'



from dataclasses import dataclass

from px_settings.contrib.django import settings as s
from wc_bankid_nbu import SignerSettings as SSBase


__all__ = (
    'SignerSettings',
    'signer_settings',
)


@s('WC_BANKID_NBU_SIGNER')
@dataclass
class SignerSettings(SSBase):
    pass


signer_settings = SignerSettings()



from social_core.backends.oauth import BaseOAuth2
from wc_bankid_nbu import (
    PersonalPhysicalDataQuery, APIClient, PersonalInfoDTO,
    TAddress, TDocument, FPerson, FAddress, FDocument,
)


__all__ = 'BankIDNBUBaseBackend',


class BankIDNBUBaseBackend(BaseOAuth2):
    """BankID NBU OAuth authentication backend"""

    name = 'bankid-nbu'
    SCOPE_SEPARATOR = ','
    EXTRA_DATA = [('id', 'id'), ('expires', 'expires')]

    client: APIClient
    PERSONAL_DATA_QUERY = (
        PersonalPhysicalDataQuery()
        .fields(
            FPerson.FIRST_NAME, FPerson.MIDDLE_NAME, FPerson.LAST_NAME,
            FPerson.BIRTH_DAY, FPerson.SEX, FPerson.SOCIAL_STATUS,

            FPerson.PHONE, FPerson.EMAIL, FPerson.RNOKPP,
            FPerson.CLIENT_ID, FPerson.CLIENT_ID_TEXT,
            FPerson.IS_HIGH_RISK, FPerson.IS_PEPS, FPerson.IS_RESTRICTED,
            FPerson.IS_TERRORIST,

            FPerson.IS_UA_RESIDENT,
        )
        .address(
            TAddress.FACTUAL,
            FAddress.COUNTRY, FAddress.REGION, FAddress.DISTRICT, FAddress.CITY,
            FAddress.STREET, FAddress.STREET_NUMBER, FAddress.FLAT_NUMBER,
        )
        .address(
            TAddress.JURIDICAL,
            FAddress.COUNTRY, FAddress.REGION, FAddress.DISTRICT, FAddress.CITY,
            FAddress.STREET, FAddress.STREET_NUMBER, FAddress.FLAT_NUMBER,
        )
        .document(
            TDocument.PASSPORT_BOOK,
            FDocument.TYPE,
            FDocument.SERIES, FDocument.NUMBER,
            FDocument.ISSUED_BY, FDocument.ISSUED_BY_COUNTRY,
            FDocument.ISSUED_AT, FDocument.EXPIRED_AT,
        )
        .document(
            TDocument.PASSPORT_ID,
            FDocument.TYPE,
            FDocument.SERIES, FDocument.NUMBER,
            FDocument.ISSUED_BY, FDocument.ISSUED_BY_COUNTRY,
            FDocument.ISSUED_AT, FDocument.EXPIRED_AT,
        )
    )

    def authorization_url(self):
        return self.client.AUTHORIZATION_URL

    def access_token_url(self):
        return self.client.ACCESS_TOKEN_URL

    def refresh_token_url(self):
        return self.client.REFRESH_TOKEN_URL

    def get_user_id(self, details, response):
        info = (response.get('data') or {}).get('info') or {}

        return info.get('rnokpp') or info.get('email')

    def get_user_details(self, response):
        """Return user details from BankID NBU account"""
        info: PersonalInfoDTO = response['data']['info']

        return {
            'username': info.get('email') or info.get('rnokpp') or info.get('phone'),
            'email': info.get('email') or '',
            'first_name': info.get('first_name') or '',
            'last_name': info.get('last_name') or '',
        }

    def user_data(self, access_token: str, *args, **kwargs):
        data = self.client.get_personal_data(
            self.PERSONAL_DATA_QUERY, access_token
        )

        return {'data': data['data']}



import os
import re
import setuptools


with open('README.md', 'r') as fh:
    long_description = fh.read()


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()

    return re.search('__version__ = [\'"]([^\'"]+)[\'"]', init_py).group(1)


version = get_version('wc_bankid_nbu')


setuptools.setup(
    name='wc-bankid-nbu',
    version=version,
    author='WebCase',
    author_email='info@webcase.studio',
    license='MIT License',
    description='SDK for NMU\'s BankID authorization.',
    include_package_data=True,
    install_requires=(
        'requests>=2.0.0,<3.0.0',
    ),
    extras_require={
        'dev': (
            'pytest>=6.0,<7.0',
            'pytest-watch>=4.2,<5.0',
            'twine',
        ),
    },
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=setuptools.find_packages(exclude=(
        'tests', 'tests.*',
        'experiments', 'pilot',
    )),
    python_requires='>=3.6',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',

        'Programming Language :: Python :: 3',

        'Intended Audience :: Developers',
        'Topic :: Utilities',

        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
)



__all__ = (
    'IDError',
    'SignerError',
    'SignerNotLoaded',
    'SignerPrivateKeyNotRead',
)


class IDError(Exception):
    pass


class SignerError(IDError):
    pass


class SignerNotLoaded(SignerError):
    pass


class SignerPrivateKeyNotRead(SignerError):
    pass



from typing import Dict, List, Sequence, Tuple, Union
from functools import partial
from datetime import date, datetime
from .dtos import AddressDTO, PersonalDataDTO, PersonalInfoDTO


__all__ = 'personal_data_response_to_dto',


Pairs = Sequence[Tuple[str, str]]
Transforms = Dict[str, callable]


def to_boolean(value: Union[str, int, bool]):
    return (
        value == '1'
        if isinstance(value, str)
        else
        value == 1
        if isinstance(value, int)
        else
        bool(value)
    )


def to_date(value: Union[str, int]):
    return (
        date.fromtimestamp(value)
        if isinstance(value, int)
        else
        datetime.strptime(value, '%d.%m.%Y').date()
        if isinstance(value, str)
        else
        None
    )


def remap_names(names: dict, value: str, default_key: str = '__default__'):
    return names[value] if value in names else names.get(default_key)


def generic_transformer(pairs: Pairs, transforms: Transforms, passed: dict) -> dict:
    result = {}

    for field, source in pairs:
        if source in passed:
            data = passed[source]

            if data is None:
                continue

            if field in transforms:
                data = transforms[field](data)

            result[field] = data

    return result


info_transformer = partial(
    generic_transformer,
    (
        ('last_name', 'lastName'),
        ('first_name', 'firstName'),
        ('middle_name', 'middleName'),

        ('phone', 'phone'),
        ('email', 'email'),

        ('rnokpp', 'inn'),

        ('client_id', 'clId'),
        ('client_id_text', 'clIdText'),

        ('birth_day', 'birthDay'),
        ('sex', 'sex'),
        ('social_status', 'socStatus'),

        ('is_peps', 'flagPEPs'),
        ('is_terrorist', 'flagPersonTerror'),
        ('is_restricted', 'flagRestriction'),
        ('is_high_risk', 'flagTopLevelRisk'),
        ('is_ua_resident', 'uaResident'),
    ),
    {
        # FIXME: It must return date type
        # 'birth_day': to_date,

        'is_peps': to_boolean,
        'is_terrorist': to_boolean,
        'is_restricted': to_boolean,
        'is_high_risk': to_boolean,
        'is_ua_resident': to_boolean,
    },
)
address_transformer = partial(
    generic_transformer,
    (
        ('type', 'type'),
        ('country', 'country'),
        ('state', 'state'),
        ('area', 'area'),
        ('city', 'city'),
        ('street', 'street'),
        ('street_number', 'houseNo'),
        ('flat_number', 'flatNo'),
    ),
    {},
)
document_transformer = partial(
    generic_transformer,
    (
        ('type', 'type'),
        ('type', 'typeName'),
        ('series', 'series'),
        ('number', 'number'),
        ('issued_by', 'issue'),
        ('issued_by_country', 'issueCountryIso2'),
        ('issued_at', 'dateIssue'),
        ('expired_at', 'dateExpiration'),
    ),
    {
        'type': partial(remap_names, {
            'passport': 'passport_book',
            'idpassport': 'passport_id',
            'zpassport': 'passport_international',
            'ident': 'identity',
        })
    },
)


def personal_data_response_to_dto(response: dict) -> PersonalDataDTO:
    result: PersonalDataDTO = {}

    result['info'] = info_transformer(response)

    addresses = result['addresses'] = []

    for address in response.get('addresses') or []:
        addresses.append(address_transformer(address))

    documents = result['documents'] = []

    for document in response.get('documents') or []:
        documents.append(document_transformer(document))

    return result
